@@ -12,7 +12,15 @@ from abc import ABCMeta
 from collections import OrderedDict, defaultdict
 from decimal import Decimal
 from enum import Enum
-from typing import Any, DefaultDict, Generic, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    Any, DefaultDict, Dict, Generic, List, Optional, Sequence, Tuple, Type, TypeVar, Union,
+)
+
+if sys.version_info >= (3, 10):
+    from typing import TypeGuard  # noqa: F401
+else:
+    from typing_extensions import TypeGuard  # noqa: F401
+
 
 import inflection
 import uritemplate
@@ -34,18 +42,24 @@ from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
 from rest_framework import exceptions, fields, mixins, serializers, versioning
 from rest_framework.compat import unicode_http_header
+from rest_framework.fields import empty
 from rest_framework.settings import api_settings
 from rest_framework.test import APIRequestFactory
+from rest_framework.utils.encoders import JSONEncoder
 from rest_framework.utils.mediatypes import _MediaType
 from rest_framework.utils.serializer_helpers import ReturnDict, ReturnList
 from uritemplate import URITemplate
 
-from drf_spectacular.drainage import cache, error, warn
+from drf_spectacular.drainage import cache, error, get_override, warn
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.types import (
     DJANGO_PATH_CONVERTER_MAPPING, OPENAPI_TYPE_MAPPING, PYTHON_TYPE_MAPPING, OpenApiTypes,
+    _KnownPythonTypes,
+)
+from drf_spectacular.utils import (
+    OpenApiExample, OpenApiParameter, _FieldType, _ListSerializerType, _ParameterLocationType,
+    _SchemaType, _SerializerType,
 )
-from drf_spectacular.utils import OpenApiParameter
 
 try:
     from django.db.models.enums import Choices  # only available in Django>3
@@ -55,9 +69,9 @@ except ImportError:
 
 # types.UnionType was added in Python 3.10 for new PEP 604 pipe union syntax
 if hasattr(types, 'UnionType'):
-    UNION_TYPES: Tuple[Any, ...] = (typing.Union, types.UnionType)  # type: ignore
+    UNION_TYPES: Tuple[Any, ...] = (Union, types.UnionType)
 else:
-    UNION_TYPES = (typing.Union,)
+    UNION_TYPES = (Union,)
 
 LITERAL_TYPES: Tuple[Any, ...] = ()
 TYPED_DICT_META_TYPES: Tuple[Any, ...] = ()
@@ -77,9 +91,9 @@ except ImportError:
     pass
 
 if sys.version_info >= (3, 8):
-    CACHED_PROPERTY_FUNCS = (functools.cached_property, cached_property)  # type: ignore
+    CACHED_PROPERTY_FUNCS = (functools.cached_property, cached_property)
 else:
-    CACHED_PROPERTY_FUNCS = (cached_property,)  # type: ignore
+    CACHED_PROPERTY_FUNCS = (cached_property,)
 
 T = TypeVar('T')
 
@@ -101,41 +115,42 @@ def force_instance(serializer_or_field):
         return serializer_or_field
 
 
-def is_serializer(obj) -> bool:
-    from drf_spectacular.serializers import OpenApiSerializerExtension
+def is_serializer(obj, strict=False) -> TypeGuard[_SerializerType]:
+
+    from drf_spectacular.extensions import OpenApiSerializerExtension
     return (
         isinstance(force_instance(obj), serializers.BaseSerializer)
-        or bool(OpenApiSerializerExtension.get_match(obj))
+        or (bool(OpenApiSerializerExtension.get_match(obj)) and not strict)
     )
 
 
-def is_list_serializer(obj) -> bool:
+def is_list_serializer(obj: Any) -> TypeGuard[_ListSerializerType]:
     return isinstance(force_instance(obj), serializers.ListSerializer)
 
 
-def get_list_serializer(obj):
+def get_list_serializer(obj: Any):
     return force_instance(obj) if is_list_serializer(obj) else get_class(obj)(many=True, context=obj.context)
 
 
 def is_list_serializer_customized(obj) -> bool:
     return (
-        is_serializer(obj)
+        is_serializer(obj, strict=True)
         and get_class(get_list_serializer(obj)).to_representation  # type: ignore
         is not serializers.ListSerializer.to_representation
     )
 
 
-def is_basic_serializer(obj) -> bool:
+def is_basic_serializer(obj: Any) -> TypeGuard[_SerializerType]:
     return is_serializer(obj) and not is_list_serializer(obj)
 
 
-def is_field(obj):
+def is_field(obj: Any) -> TypeGuard[_FieldType]:
     # make sure obj is a serializer field and nothing else.
     # guard against serializers because BaseSerializer(Field)
     return isinstance(force_instance(obj), fields.Field) and not is_serializer(obj)
 
 
-def is_basic_type(obj, allow_none=True):
+def is_basic_type(obj: Any, allow_none=True) -> TypeGuard[_KnownPythonTypes]:
     if not isinstance(obj, collections.abc.Hashable):
         return False
     if not allow_none and (obj is None or obj is OpenApiTypes.NONE):
@@ -143,22 +158,22 @@ def is_basic_type(obj, allow_none=True):
     return obj in get_openapi_type_mapping() or obj in PYTHON_TYPE_MAPPING
 
 
-def is_patched_serializer(serializer, direction):
+def is_patched_serializer(serializer, direction) -> bool:
     return bool(
         spectacular_settings.COMPONENT_SPLIT_PATCH
-        and serializer.partial
-        and not serializer.read_only
+        and getattr(serializer, 'partial', None)
+        and not getattr(serializer, 'read_only', None)
         and not (spectacular_settings.COMPONENT_SPLIT_REQUEST and direction == 'response')
     )
 
 
-def is_trivial_string_variation(a: str, b: str):
+def is_trivial_string_variation(a: str, b: str) -> bool:
     a = (a or '').strip().lower().replace(' ', '_').replace('-', '_')
     b = (b or '').strip().lower().replace(' ', '_').replace('-', '_')
     return a == b
 
 
-def assert_basic_serializer(serializer):
+def assert_basic_serializer(serializer) -> None:
     assert is_basic_serializer(serializer), (
         f'internal assumption violated because we expected a basic serializer here and '
         f'instead got a "{serializer}". This may be the result of another app doing '
@@ -206,9 +221,9 @@ def get_view_model(view, emit_warnings=True):
             )
 
 
-def get_doc(obj):
+def get_doc(obj) -> str:
     """ get doc string with fallback on obj's base classes (ignoring DRF documentation). """
-    def post_cleanup(doc: str):
+    def post_cleanup(doc: str) -> str:
         # also clean up trailing whitespace for each line
         return '\n'.join(line.rstrip() for line in doc.rstrip().split('\n'))
 
@@ -230,7 +245,7 @@ def get_doc(obj):
     return ''
 
 
-def get_type_hints(obj):
+def get_type_hints(obj) -> Dict[str, Any]:
     """ unpack wrapped partial object and use actual func object """
     if isinstance(obj, functools.partial):
         obj = obj.func
@@ -264,7 +279,7 @@ def build_generic_type():
         return {'type': 'object', 'additionalProperties': {}}
 
 
-def build_basic_type(obj):
+def build_basic_type(obj: Union[_KnownPythonTypes, OpenApiTypes]) -> Optional[_SchemaType]:
     """
     resolve either enum or actual type and yield schema template for modification
     """
@@ -280,7 +295,7 @@ def build_basic_type(obj):
         return dict(openapi_type_mapping[OpenApiTypes.STR])
 
 
-def build_array_type(schema, min_length=None, max_length=None):
+def build_array_type(schema: _SchemaType, min_length=None, max_length=None) -> _SchemaType:
     schema = {'type': 'array', 'items': schema}
     if min_length is not None:
         schema['minLength'] = min_length
@@ -290,12 +305,12 @@ def build_array_type(schema, min_length=None, max_length=None):
 
 
 def build_object_type(
-        properties=None,
+        properties: Optional[_SchemaType] = None,
         required=None,
-        description=None,
+        description: Optional[str] = None,
         **kwargs
-):
-    schema = {'type': 'object'}
+) -> _SchemaType:
+    schema: _SchemaType = {'type': 'object'}
     if description:
         schema['description'] = description.strip()
     if properties:
@@ -308,7 +323,7 @@ def build_object_type(
     return schema
 
 
-def build_media_type_object(schema, examples=None, encoding=None):
+def build_media_type_object(schema, examples=None, encoding=None) -> _SchemaType:
     media_type_object = {'schema': schema}
     if examples:
         media_type_object['examples'] = examples
@@ -317,12 +332,12 @@ def build_media_type_object(schema, examples=None, encoding=None):
     return media_type_object
 
 
-def build_examples_list(examples):
+def build_examples_list(examples: Sequence[OpenApiExample]) -> _SchemaType:
     schema = {}
     for example in examples:
         normalized_name = inflection.camelize(example.name.replace(' ', '_'))
         sub_schema = {}
-        if example.value:
+        if example.value is not empty:
             sub_schema['value'] = example.value
         if example.external_value:
             sub_schema['externalValue'] = example.external_value
@@ -337,9 +352,9 @@ def build_examples_list(examples):
 
 
 def build_parameter_type(
-        name,
-        schema,
-        location,
+        name: str,
+        schema: _SchemaType,
+        location: _ParameterLocationType,
         required=False,
         description=None,
         enum=None,
@@ -351,7 +366,7 @@ def build_parameter_type(
         allow_blank=True,
         examples=None,
         extensions=None,
-):
+) -> _SchemaType:
     irrelevant_field_meta = ['readOnly', 'writeOnly']
     if location == OpenApiParameter.PATH:
         irrelevant_field_meta += ['nullable', 'default']
@@ -393,11 +408,11 @@ def build_parameter_type(
     return schema
 
 
-def build_choice_field(field):
+def build_choice_field(field) -> _SchemaType:
     choices = list(OrderedDict.fromkeys(field.choices))  # preserve order and remove duplicates
 
     if all(isinstance(choice, bool) for choice in choices):
-        type = 'boolean'
+        type: Optional[str] = 'boolean'
     elif all(isinstance(choice, int) for choice in choices):
         type = 'integer'
     elif all(isinstance(choice, (int, float, Decimal)) for choice in choices):  # `number` includes `integer`
@@ -408,12 +423,12 @@ def build_choice_field(field):
     else:
         type = None
 
-    if field.allow_blank:
+    if field.allow_blank and '' not in choices:
         choices.append('')
-    if field.allow_null:
+    if field.allow_null and None not in choices:
         choices.append(None)
 
-    schema = {
+    schema: _SchemaType = {
         # The value of `enum` keyword MUST be an array and SHOULD be unique.
         # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.20
         'enum': choices
@@ -429,6 +444,8 @@ def build_choice_field(field):
     if spectacular_settings.ENUM_GENERATE_CHOICE_DESCRIPTION:
         schema['description'] = build_choice_description_list(field.choices.items())
 
+    schema['x-spec-enum-id'] = list_hash([(k, v) for k, v in field.choices.items() if k not in ('', None)])
+
     return schema
 
 
@@ -460,14 +477,14 @@ def build_bearer_security_scheme_object(header_name, token_prefix, bearer_format
         }
 
 
-def build_root_object(paths, components, version):
+def build_root_object(paths, components, version) -> _SchemaType:
     settings = spectacular_settings
     if settings.VERSION and version:
         version = f'{settings.VERSION} ({version})'
     else:
         version = settings.VERSION or version or ''
     root = {
-        'openapi': '3.0.3',
+        'openapi': settings.OAS_VERSION,
         'info': {
             'title': settings.TITLE,
             'version': version,
@@ -494,17 +511,37 @@ def build_root_object(paths, components, version):
     return root
 
 
-def safe_ref(schema):
+def safe_ref(schema: _SchemaType) -> _SchemaType:
     """
     ensure that $ref has its own context and does not remove potential sibling
-    entries when $ref is substituted.
+    entries when $ref is substituted. also remove useless singular "allOf" .
     """
     if '$ref' in schema and len(schema) > 1:
         return {'allOf': [{'$ref': schema.pop('$ref')}], **schema}
+    if 'allOf' in schema and len(schema) == 1 and len(schema['allOf']) == 1:
+        return schema['allOf'][0]
     return schema
 
 
-def append_meta(schema, meta):
+def append_meta(schema: _SchemaType, meta: _SchemaType) -> _SchemaType:
+    if spectacular_settings.OAS_VERSION.startswith('3.1'):
+        schema_nullable = meta.pop('nullable', None)
+        meta_nullable = schema.pop('nullable', None)
+
+        if schema_nullable or meta_nullable:
+            if 'type' in schema:
+                schema['type'] = [schema['type'], 'null']
+            elif '$ref' in schema:
+                schema = {'oneOf': [schema, {'type': 'null'}]}
+            else:
+                assert False, 'Invalid nullable case'  # pragma: no cover
+
+        # these two aspects were merged in OpenAPI 3.1
+        if "exclusiveMinimum" in schema and "minimum" in schema:
+            schema["exclusiveMinimum"] = schema.pop("minimum")
+        if "exclusiveMaximum" in schema and "maximum" in schema:
+            schema["exclusiveMaximum"] = schema.pop("maximum")
+
     return safe_ref({**schema, **meta})
 
 
@@ -530,9 +567,9 @@ def _follow_field_source(model, path: List[str]):
         elif isinstance(field_or_property, ReverseOneToOneDescriptor):
             return field_or_property.related.target_field  # o2o reverse
         elif isinstance(field_or_property, ReverseManyToOneDescriptor):
-            return field_or_property.rel.target_field  # type: ignore # foreign reverse
+            return field_or_property.rel.target_field  # foreign reverse
         elif isinstance(field_or_property, ForwardManyToOneDescriptor):
-            return field_or_property.field.target_field  # type: ignore # o2o & foreign forward
+            return field_or_property.field.target_field  # o2o & foreign forward
         else:
             field = model._meta.get_field(path[0])
             if isinstance(field, ForeignObjectRel):
@@ -652,20 +689,20 @@ class ResolvedComponent:
         return bool(self.name and self.type and self.object)
 
     @property
-    def key(self):
+    def key(self) -> Tuple[str, str]:
         return self.name, self.type
 
     @property
-    def ref(self) -> dict:
+    def ref(self) -> _SchemaType:
         assert self.__bool__()
         return {'$ref': f'#/components/{self.type}/{self.name}'}
 
 
 class ComponentRegistry:
-    def __init__(self):
-        self._components = {}
+    def __init__(self) -> None:
+        self._components: Dict[Tuple[str, str], ResolvedComponent] = {}
 
-    def register(self, component: ResolvedComponent):
+    def register(self, component: ResolvedComponent) -> None:
         if component in self:
             warn(
                 f'trying to re-register a {component.type} component with name '
@@ -674,7 +711,7 @@ class ComponentRegistry:
             )
         self._components[component.key] = component
 
-    def register_on_missing(self, component: ResolvedComponent):
+    def register_on_missing(self, component: ResolvedComponent) -> None:
         if component not in self:
             self._components[component.key] = component
 
@@ -687,7 +724,11 @@ class ComponentRegistry:
         query_class = query_obj if inspect.isclass(query_obj) else query_obj.__class__
         registry_class = query_obj if inspect.isclass(registry_obj) else registry_obj.__class__
 
-        if query_class != registry_class:
+        suppress_collision_warning = (
+            get_override(registry_class, 'suppress_collision_warning', False)
+            or get_override(query_class, 'suppress_collision_warning', False)
+        )
+        if query_class != registry_class and not suppress_collision_warning:
             warn(
                 f'Encountered 2 components with identical names "{component.name}" and '
                 f'different classes {query_class} and {registry_class}. This will very '
@@ -695,7 +736,7 @@ class ComponentRegistry:
             )
         return True
 
-    def __getitem__(self, key):
+    def __getitem__(self, key) -> ResolvedComponent:
         if isinstance(key, ResolvedComponent):
             key = key.key
         return self._components[key]
@@ -705,8 +746,8 @@ class ComponentRegistry:
             key = key.key
         del self._components[key]
 
-    def build(self, extra_components) -> dict:
-        output: DefaultDict[str, dict] = defaultdict(dict)
+    def build(self, extra_components) -> _SchemaType:
+        output: DefaultDict[str, _SchemaType] = defaultdict(dict)
         # build tree from flat registry
         for component in self._components.values():
             output[component.type][component.name] = component.schema
@@ -722,7 +763,7 @@ class ComponentRegistry:
 
 
 class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
-    _registry: List[T] = []
+    _registry: List[Type[T]] = []
     target_class: Union[None, str, Type[object]] = None
     match_subclasses = False
     priority = 0
@@ -757,14 +798,18 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
             cls.target_class = None
 
     @classmethod
-    def _matches(cls, target) -> bool:
+    def _matches(cls, target: Any) -> bool:
         if isinstance(cls.target_class, str):
             cls._load_class()
 
         if cls.target_class is None:
             return False  # app not installed
         elif cls.match_subclasses:
-            return issubclass(get_class(target), cls.target_class)  # type: ignore
+            # Targets may trigger customized check through __subclasscheck__. Attempt to be more robust
+            try:
+                return issubclass(get_class(target), cls.target_class)  # type: ignore
+            except TypeError:
+                return False
         else:
             return get_class(target) == cls.target_class
 
@@ -776,7 +821,7 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
         return None
 
 
-def deep_import_string(string):
+def deep_import_string(string: str) -> Any:
     """ augmented import from string, e.g. MODULE.CLASS/OBJECT.ATTRIBUTE """
     try:
         return import_string(string)
@@ -805,11 +850,12 @@ def load_enum_name_overrides():
         if inspect.isclass(choices) and issubclass(choices, Choices):
             choices = choices.choices
         if inspect.isclass(choices) and issubclass(choices, Enum):
-            choices = [c.value for c in choices]
+            choices = [(c.value, c.name) for c in choices]
         normalized_choices = []
         for choice in choices:
             # Allow None values in the simple values list case
             if isinstance(choice, str) or choice is None:
+                # TODO warning
                 normalized_choices.append((choice, choice))  # simple choice list
             elif isinstance(choice[1], (list, tuple)):
                 normalized_choices.extend(choice[1])  # categorized nested choices
@@ -818,7 +864,9 @@ def load_enum_name_overrides():
 
         # Get all of choice values that should be used in the hash, blank and None values get excluded
         # in the post-processing hook for enum overrides, so we do the same here to ensure the hashes match
-        hashable_values = [value for value, _ in normalized_choices if value not in ['', None]]
+        hashable_values = [
+            (value, label) for value, label in normalized_choices if value not in ['', None]
+        ]
         overrides[list_hash(hashable_values)] = name
 
     if len(spectacular_settings.ENUM_NAME_OVERRIDES) != len(overrides):
@@ -829,8 +877,8 @@ def load_enum_name_overrides():
     return overrides
 
 
-def list_hash(lst):
-    return hashlib.sha256(json.dumps(list(lst), sort_keys=True).encode()).hexdigest()
+def list_hash(lst: Any) -> str:
+    return hashlib.sha256(json.dumps(list(lst), sort_keys=True, cls=JSONEncoder).encode()).hexdigest()[:16]
 
 
 def anchor_pattern(pattern: str) -> str:
@@ -930,7 +978,7 @@ def resolve_regex_path_parameter(path_regex, variable):
     return None
 
 
-def is_versioning_supported(versioning_class):
+def is_versioning_supported(versioning_class) -> bool:
     return issubclass(versioning_class, (
         versioning.URLPathVersioning,
         versioning.NamespaceVersioning,
@@ -938,7 +986,7 @@ def is_versioning_supported(versioning_class):
     ))
 
 
-def operation_matches_version(view, requested_version):
+def operation_matches_version(view, requested_version) -> bool:
     try:
         version, _ = view.determine_version(view.request, **view.kwargs)
     except exceptions.NotAcceptable:
@@ -1004,7 +1052,7 @@ def modify_media_types_for_versioning(view, media_types: List[str]) -> List[str]
     ]
 
 
-def analyze_named_regex_pattern(path):
+def analyze_named_regex_pattern(path: str) -> Dict[str, str]:
     """ safely extract named groups and their pattern from given regex pattern """
     result = {}
     stack = 0
@@ -1285,7 +1333,7 @@ def resolve_type_hint(hint):
         raise UnableToProceedError()
 
 
-def whitelisted(obj: object, classes: Optional[List[Type[object]]], exact=False):
+def whitelisted(obj: object, classes: Optional[List[Type[object]]], exact=False) -> bool:
     if classes is None:
         return True
     if exact:
@@ -1303,17 +1351,18 @@ def build_mocked_view(method: str, path: str, extend_schema_decorator, registry)
 
     # emulate what Generator would do to setup schema generation.
     view_callable = TmpView.as_view()
-    view = view_callable.cls()  # type: ignore
+    view: views.APIView = view_callable.cls()
     view.request = spectacular_settings.GET_MOCK_REQUEST(
         method.upper(), path, view, None
     )
     view.kwargs = {}
     # prepare AutoSchema with "init" values as if get_operation() was called
-    view.schema.registry = registry
-    view.schema.path = path
-    view.schema.path_regex = path
-    view.schema.path_prefix = ''
-    view.schema.method = method.upper()
+    schema: Any = view.schema
+    schema.registry = registry
+    schema.path = path
+    schema.path_regex = path
+    schema.path_prefix = ''
+    schema.method = method.upper()
     return view
 
 
@@ -1323,6 +1372,10 @@ def build_listed_example_value(value: Any, paginator, direction):
 
     sentinel = object()
     schema = paginator.get_paginated_response_schema(sentinel)
+
+    if schema is sentinel:
+        return [value]
+
     try:
         return {
             field_name: [value] if field_schema is sentinel else field_schema['example']
