@@ -44,6 +44,11 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     # accurately modeled when request and response components are separated.
     'ENFORCE_NON_BLANK_FIELDS': False,
 
+    # This version string will end up the in schema header. The default OpenAPI
+    # version is 3.0.3, which is heavily tested. We now also support 3.1.0,
+    # which contains the same features and a few mandatory, but minor changes.
+    'OAS_VERSION': '3.0.3',
+
     # Configuration for serving a schema subset with SpectacularAPIView
     'SERVE_URLCONF': None,
     # complete public schema or a subset based on the requesting user
@@ -83,7 +88,8 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     'APPEND_PATHS': {},
     'APPEND_COMPONENTS': {},
 
-    # DISCOURAGED - please don't use this anymore as it has tricky implications that
+    # STRONGLY DISCOURAGED (with the exception for the djangorestframework-api-key library)
+    # please don't use this anymore as it has tricky implications that
     # are hard to get right. For authentication, OpenApiAuthenticationExtension are
     # strongly preferred because they are more robust and easy to write.
     # However if used, the list of methods is appended to every endpoint in the schema!
@@ -112,6 +118,8 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     'ENUM_NAME_OVERRIDES': {},
     # Adds "blank" and "null" enum choices where appropriate. disable on client generation issues
     'ENUM_ADD_EXPLICIT_BLANK_NULL_CHOICE': True,
+    # Add/Append a list of (``choice value`` - choice name) to the enum description string.
+    'ENUM_GENERATE_CHOICE_DESCRIPTION': True,
 
     # function that returns a list of all classes that should be excluded from doc string extraction
     'GET_LIB_DOC_EXCLUDES': 'drf_spectacular.plumbing.get_lib_doc_excludes',
@@ -154,15 +162,17 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     # will likely fix most issues, though you are free to choose any name.
     "DEFAULT_QUERY_MANAGER": 'objects',
 
-    # Controls which authentication methods are exposed in the schema. If not empty, will hide
+    # Controls which authentication methods are exposed in the schema. If not None, will hide
     # authentication classes that are not contained in the whitelist. Use full import paths
-    # like ['rest_framework.authentication.TokenAuthentication', ...]
-    'AUTHENTICATION_WHITELIST': [],
+    # like ['rest_framework.authentication.TokenAuthentication', ...].
+    # Empty list ([]) will hide all authentication methods. The default None will show all.
+    'AUTHENTICATION_WHITELIST': None,
     # Controls which parsers are exposed in the schema. Works analog to AUTHENTICATION_WHITELIST.
-    'PARSER_WHITELIST': [],
+    # List of allowed parsers or None to allow all.
+    'PARSER_WHITELIST': None,
     # Controls which renderers are exposed in the schema. Works analog to AUTHENTICATION_WHITELIST.
-    # rest_framework.renderers.BrowsableAPIRenderer is ignored by default if whitelist is empty
-    'RENDERER_WHITELIST': [],
+    # rest_framework.renderers.BrowsableAPIRenderer is ignored by default if whitelist is None
+    'RENDERER_WHITELIST': None,
 
     # Option for turning off error and warn messages
     'DISABLE_ERRORS_AND_WARNINGS': False,
