@@ -2,6 +2,8 @@ import inspect
 import sys
 from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, TypeVar, Union
 
+from django.utils.functional import Promise
+
 # direct import due to https://github.com/microsoft/pyright/issues/3025
 if sys.version_info >= (3, 8):
     from typing import Final, Literal
@@ -9,7 +11,7 @@ else:
     from typing_extensions import Final, Literal
 
 from rest_framework.fields import Field, empty
-from rest_framework.serializers import Serializer
+from rest_framework.serializers import ListSerializer, Serializer
 from rest_framework.settings import api_settings
 
 from drf_spectacular.drainage import (
@@ -17,9 +19,12 @@ from drf_spectacular.drainage import (
 )
 from drf_spectacular.types import OpenApiTypes, _KnownPythonTypes
 
+_ListSerializerType = Union[ListSerializer, Type[ListSerializer]]
 _SerializerType = Union[Serializer, Type[Serializer]]
 _FieldType = Union[Field, Type[Field]]
 _ParameterLocationType = Literal['query', 'path', 'header', 'cookie']
+_StrOrPromise = Union[str, Promise]
+_SchemaType = Dict[str, Any]
 Direction = Literal['request', 'response']
 
 
@@ -79,17 +84,28 @@ class PolymorphicProxySerializer(Serializer):
             ],
             resource_type_field_name: Optional[str],
             many: Optional[bool] = None,
+            **kwargs
     ):
         self.component_name = component_name
         self.serializers = serializers
         self.resource_type_field_name = resource_type_field_name
         if self._many is False:  # type: ignore[attr-defined]
             set_override(self, 'many', False)
-        super().__init__()
+        # retain kwargs in context for potential anonymous re-init with many=True
+        kwargs.setdefault('context', {}).update({
+            'component_name': component_name,
+            'serializers': serializers,
+            'resource_type_field_name': resource_type_field_name
+        })
+        super().__init__(**kwargs)
 
     def __new__(cls, *args, **kwargs):
         many = kwargs.pop('many', None)
         if many is True:
+            context = kwargs.get('context', {})
+            for arg in ['component_name', 'serializers', 'resource_type_field_name']:
+                if arg in context:
+                    kwargs[arg] = context.pop(arg)  # re-apply retained args
             instance = cls.many_init(*args, **kwargs)
         else:
             instance = super().__new__(cls, *args, **kwargs)
@@ -147,10 +163,10 @@ class OpenApiExample(OpenApiSchemaBase):
     def __init__(
             self,
             name: str,
-            value: Any = None,
+            value: Any = empty,
             external_value: str = '',
-            summary: str = '',
-            description: str = '',
+            summary: _StrOrPromise = '',
+            description: _StrOrPromise = '',
             request_only: bool = False,
             response_only: bool = False,
             parameter_only: Optional[Tuple[str, _ParameterLocationType]] = None,
@@ -188,10 +204,10 @@ class OpenApiParameter(OpenApiSchemaBase):
     def __init__(
             self,
             name: str,
-            type: Union[_SerializerType, _KnownPythonTypes, OpenApiTypes, dict] = str,
+            type: Union[_SerializerType, _KnownPythonTypes, OpenApiTypes, _SchemaType] = str,
             location: _ParameterLocationType = QUERY,
             required: bool = False,
-            description: str = '',
+            description: _StrOrPromise = '',
             enum: Optional[Sequence[Any]] = None,
             pattern: Optional[str] = None,
             deprecated: bool = False,
@@ -236,7 +252,7 @@ class OpenApiResponse(OpenApiSchemaBase):
     def __init__(
             self,
             response: Any = None,
-            description: str = '',
+            description: _StrOrPromise = '',
             examples: Optional[Sequence[OpenApiExample]] = None
     ):
         self.response = response
@@ -291,7 +307,7 @@ class OpenApiCallback(OpenApiSchemaBase):
     """
     def __init__(
             self,
-            name: str,
+            name: _StrOrPromise,
             path: str,
             decorator: Union[Callable[[F], F], Dict[str, Callable[[F], F]], Dict[str, Any]],
     ):
@@ -306,13 +322,13 @@ def extend_schema(
         request: Any = empty,
         responses: Any = empty,
         auth: Optional[Sequence[str]] = None,
-        description: Optional[str] = None,
-        summary: Optional[str] = None,
+        description: Optional[_StrOrPromise] = None,
+        summary: Optional[_StrOrPromise] = None,
         deprecated: Optional[bool] = None,
         tags: Optional[Sequence[str]] = None,
         filters: Optional[bool] = None,
-        exclude: bool = False,
-        operation: Optional[Dict] = None,
+        exclude: Optional[bool] = None,
+        operation: Optional[_SchemaType] = None,
         methods: Optional[Sequence[str]] = None,
         versions: Optional[Sequence[str]] = None,
         examples: Optional[Sequence[OpenApiExample]] = None,
@@ -403,12 +419,15 @@ def extend_schema(
             def get_operation(self, path, path_regex, path_prefix, method, registry):
                 self.method = method.upper()
 
-                if exclude and is_in_scope(self):
-                    return None
                 if operation is not None and is_in_scope(self):
                     return operation
                 return super().get_operation(path, path_regex, path_prefix, method, registry)
 
+            def is_excluded(self):
+                if exclude is not None and is_in_scope(self):
+                    return exclude
+                return super().is_excluded()
+
             def get_operation_id(self):
                 if operation_id and is_in_scope(self):
                     return operation_id
@@ -525,7 +544,7 @@ def extend_schema(
 
 
 def extend_schema_field(
-        field: Union[_SerializerType, _FieldType, OpenApiTypes, Dict],
+        field: Union[_SerializerType, _FieldType, OpenApiTypes, _SchemaType],
         component_name: Optional[str] = None
 ) -> Callable[[F], F]:
     """
